/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* MPIA, NEVEC - MIDI project
*
* "@(#) $Id: table_def_priv.c,v 1.12 2010/07/15 20:40:37 jaffe Exp $"
*
* who        when      what
* ---------  --------  ----------------------------------------------
* jdejong    2002-05-08 imported
*
*/
/* $Id: table_def_priv.c,v 1.12 2010/07/15 20:40:37 jaffe Exp $ */
#define _POSIX_SOURCE_1
#include "vltPort.h"

/*----------------------------------------------------------------------- */
/* Table definition class for FITS binary tables.                         */
/* Private functions                                                      */
/* This package creates and maintains table definition structures used to */
/* create FITS binary tables.                                             */
/* A new table type XXX requires: (mostly in table_*_def.c)               */
/*  1) a unique definition of XXX_TABLE in VLTItable.h                    */  
/*  2) prototype and definition of make_XXX_table() which includes XXX.def*/
/*     in VLTIprivate.h and table_def_priv.c                              */ 
/*  3) an XXX.def file defining the components of the table in ../include */
/*  4) add case for table type in make_table_def                          */
/*     in table_def_pub.c                                                 */
/*----------------------------------------------------------------------- */
#include "fitsio.h"
#include "VLTIprivate.h"

/**************************** private functions *******************************/
table_def* create_table_def(char  colsymb, int symcount,
			    char* extname, int nkey, char* keywords[], 
			    int keytype[], char* keycom[],
			    int ncol, char* coltype[], long colcount[],
			    int coldatatype[], char* colunits[], char* coldim[])
/*----------------------------------------------------------------------- */
/*   Make a table_def object from values                                  */
/*   Column with names containing a specified character may be repeated   */ 
/*   with the symbol replaced by integers running from 1 to a specified   */
/*   value. Other properties of the column remain the same.               */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*      nkey         The number of keywords in the table                  */
/*      keywords     Array of the keywords                                */
/*      keytype      Keyword types (defined in fitsio.h)                  */
/*      keycom       Descriptions of the keywords                         */
/*      ncol         Number of columns in the table                       */
/*      coltype      Column labels                                        */
/*      colcount     Dimensionality as 1D of the column                   */
/*      coldatatype  Data type codes of column values (defined in fitsio.h)*/
/*      colunits     Units of column data                                 */
/*      coldim       Dimensionality string ('(2,2)') for 2 or higher      */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure;                  */
/*      NULL indicates failure                                            */
/*----------------------------------------------------------------------- */
{
  long i, j, dsize;
  int hasym = 0; /* how many column names contain colsymb */
  int totalcol, *firstcol, icol;
  char tstring[FLEN_CARD], tstring2[FLEN_CARD];

  table_def *out = (table_def*)malloc (sizeof(table_def));  /* output structure */
  if (out==NULL) return out;  /* check for allocation failure */

  /* need array showing the first occurance of colsymb */
  firstcol = (int*)malloc(ncol*sizeof(int));
  for (i=0; i<ncol; i++) firstcol[i]=-1; /* init */

  /* count occurances of column names containing colsymb */
  if (colsymb!=0) {
    for (i=0; i<ncol; i++) {
      for (j=0; j<(int)strlen(coltype[i]); j++)
	if (colsymb==coltype[i][j]) { /* found one */
	  firstcol[i] = j;         /* remember location */
	  hasym++; 
	  break;
	}
    }
  } /* end of non NULL colsymb */
    
  /* how many columns in all */
  totalcol = ncol + hasym*(symcount-1);

  out->nkey = nkey;
  out->ncol = totalcol;  /* save input */
  out->extname = (char*)malloc(strlen(extname)+1);
  strcpy (out->extname, extname);

  /* create keyword structures */
  out->keywords = (char**) malloc (nkey*sizeof(char*));
  out->keytype = (int*)malloc (nkey*sizeof(int*));
  out->keycom = (char**)malloc (nkey*sizeof(char*));
  out->keydata = (void**)malloc (nkey*sizeof(void*));

  /* keywords */
  for (i=0; i<nkey; i++) {
    out->keywords[i] = (char*)malloc(strlen(keywords[i])+1);
    strcpy (out->keywords[i], keywords[i]);
    out->keycom[i] = (char*)malloc(strlen(keycom[i])+1);
    strcpy (out->keycom[i], keycom[i]);
    out->keytype[i] = keytype[i];
    if (keytype[i]==TSTRING) dsize = FLEN_VALUE; /* max. string size */
    else dsize = FITS_type_to_size(keytype[i]);  /* numeric values */
    out->keydata[i] = (void*)malloc(dsize); /* allocate keyword value storage */
  } /* end loop over keywords */

  /* create column structures */
  out->coltype =     (char**) malloc (totalcol*sizeof(char*));
  out->colcount =    (long*)  malloc (totalcol*sizeof(long));
  out->coldatatype = (int*)   malloc (totalcol*sizeof(int));
  out->colunits =    (char**) malloc (totalcol*sizeof(char*));
  out->coldim =      (char**) malloc (totalcol*sizeof(char*));

  /* columns */
  /* this pass add only the colsymb=>'1' versions */
  icol = 0;
  for (i=0; i<ncol; i++) {
    /* get column name, this pass replace colsymb with '1' */
    strcpy (tstring, coltype[i]);
    if (firstcol[i]>0) { /* replace */
      tstring[firstcol[i]] = '1';
      tstring[firstcol[i]+1] = 0; /* terminate */
      }
    out->coltype[icol] = (char*)malloc(strlen(tstring)+1);
    strcpy (out->coltype[icol], tstring);
    out->colcount[icol] = colcount[i];
    out->coldatatype[icol] = coldatatype[i];
    out->colunits[icol] = (char*)malloc(strlen(colunits[i])+1);
    strcpy (out->colunits[icol], colunits[i]);
    out->coldim[icol] = (char*)malloc(strlen(coldim[i])+1);
    strcpy (out->coldim[icol], coldim[i]);
    icol++;  /* actual column counter */
  } /* end first loop over columns */

  /* possible loop for repeated columns */
  if (hasym>0) {
    for (j=2; j<=symcount; j++) { /* loop over total repeats */
      for (i=0; i<ncol; i++) {
	if (firstcol[i]>0) { /* is this a repeated column? */
	  /* get column name, this pass replace colsymb with '1' */
	  strcpy (tstring2, coltype[i]);
	  tstring2[firstcol[i]] = 0; /* terminate */
	  sprintf (tstring,"%s%d",tstring2,(int)j); /* form name */
	  out->coltype[icol] = (char*)malloc(strlen(tstring)+1);
	  strcpy (out->coltype[icol], tstring);
	  out->colcount[icol] = colcount[i];
	  out->coldatatype[icol] = coldatatype[i];
	  out->colunits[icol] = (char*)malloc(strlen(colunits[i])+1);
	  strcpy (out->colunits[icol], colunits[i]);
	  out->coldim[icol] = (char*)malloc(strlen(coldim[i])+1);
	  strcpy (out->coldim[icol], coldim[i]);
	  icol++;  /* actual column counter */
	} /* end of if repeated column */
      } /* end of loop over columns */
    } /* end of loop over total repeats */
  } /* end optional repeat of columns */

  /* clean up */
  if (firstcol!=NULL) free (firstcol);

  return out; /* done */
} /* end create_table_def */

void get_application_keywords(fitsfile *fptr, table_def *tabledef, 
			      int *ierr)
/*----------------------------------------------------------------------- */
/*   Read any application specific keywords, add to table_def             */
/*   This is fairly obscure with cfitsio.                                 */
/*   Inputs:                                                              */
/*      fptr         cfitsio pointer to open FITS file                    */
/*   Outputs::                                                            */
/*      tabledef     Table def to add keywords to.                        */
/*      ierr         Error code, 0 = OK, else failed                      */
/*                   checked on input                                     */
/*----------------------------------------------------------------------- */
{
#define MAX_KEYWORDS_SAVED 1024   /* maximum number of application keywords*/
  long number = -1;                    /* how many keywords? */
  char* keywords[MAX_KEYWORDS_SAVED];  /* keyword names */
  int   keytype[MAX_KEYWORDS_SAVED];   /* keyword types */
  int   keysize[MAX_KEYWORDS_SAVED];   /* keyword data size in bytes */
  char* keycomm[MAX_KEYWORDS_SAVED];   /* keyword comments */
  void* keyvals[MAX_KEYWORDS_SAVED];   /* keyword values */
  int number_inc = 1;  /* number in include list */

  char* include[] ={ /* keyword include list */
    "*"
  };
  /* don't want standard keywords */
  int number_exc = 22;  /* number in exclude list */
  char* exclude[] ={ /* keyword exclude list */
    "XTENSION",
    "BITPIX",
    "NAXIS",
    "NAXIS*",
    "PCOUNT",
    "GCOUNT",
    "TFIELDS",
    "TFORM*",
    "EXTNAME",
    "EXTVER",
    "EXTLEVEL",
    "TTYPE*",
    "TUNIT*",
    "TSCAL*",
    "TZERO*",
    "TNULL*",
    "TDISP*",
    "THEAP",
    "TDIM*",
    "HISTORY",
    "COMMENT",
    "DATE"
  }; /* end of exclude list */

  int  more = 1;  /* are there more keywords to search? */
  char card[FLEN_CARD], card_key[9], card_value[FLEN_VALUE], 
    card_comm[FLEN_COMMENT], dtype;
  int  itype;
  char  dvalue[FLEN_VALUE+20];
  long  dsize, i;

  if (*ierr>0) return; /* existing error condition? */

  while (more) {
    fits_find_nextkey (fptr, include, number_inc, exclude, number_exc,
		       card, ierr);
    if (*ierr==KEY_NO_EXIST) { /* finished ? */
      *ierr = 0;
      more = 0;  /* stop looping */
      break;     /* none this time either */
    }
    if (*ierr!=0) {
      fprintf (stderr, "get_application_keywords: Error %d reading keywords\n.", 
	       *ierr);
      return;
    }

    /* extract keyword from card */
    strncpy (card_key, card, 8);
    card_key[8] = 0; /* in case */

    /* get value string and comment */
    fits_parse_value (card, card_value, card_comm, ierr);

    /* get data type */
    fits_get_keytype (card_value, &dtype, ierr);
    if (*ierr!=0) {
      fprintf (stderr, "get_application_keywords: Error %d parsing card\n %s\n.", 
	       *ierr, card);
      return;
    }

    /* get value by type */
    switch (dtype) {
    case 'C': /* Character string */
      itype = TSTRING;
      dsize = FLEN_VALUE;
      break;
    case 'L': /* Logical */
      itype = TLOGICAL;
      dsize = sizeof(int);
      break;
    case 'I': /* Integer (use int) */
      itype = TINT;
      dsize = sizeof(int);
      break;
    case 'F': /* Floating  (use double) */
      itype = TDOUBLE;
      dsize = sizeof(double);
      break;
    case 'X': /* Complex */
      itype = TCOMPLEX;
      dsize = 2*sizeof(float);
      break;
    default:
      itype = TINT;
      dsize = sizeof(int);
    } /* end switch by data type */

    /* read value */
    if (dtype=='C') { /* special handling for strings */
      fits_read_key (fptr, itype, card_key, (void*)dvalue, card_comm, ierr);
    } else { /* normal */
      fits_read_key (fptr, itype, card_key, (void*)dvalue, card_comm, ierr);
    }
    if (*ierr!=0) {
      fprintf (stderr, "get_application_keywords: Error %d reading keyword %s\n.", 
	       *ierr, card_key);
      return;
    }
    
    /* save keyword info */
    if (number>= MAX_KEYWORDS_SAVED) { /* oops - too many */
      *ierr = 16000;
      fprintf (stderr, "get_application_keywords: Too many keywords for internal array\n.");        ;
      return;
    }

    /* save entry */
    number++;
    keywords[number] = (char*)malloc(FLEN_KEYWORD);  /* keyword */
    strcpy (keywords[number], card_key);
    keytype[number] = itype;                         /* type */
    keysize[number] = dsize;                         /* size in bytes */
    keycomm[number] = (char*)malloc(FLEN_KEYWORD);   /* comment */
    strcpy (keycomm[number], card_comm);
    keyvals[number] = (void*)malloc(dsize);          /* data copy */
    memcpy ((keyvals[number]), (void*)dvalue, (size_t)dsize);

    /* remove trailing blanks from strings */
    table_deblank(keywords[number]);
    table_deblank(keycomm[number]);
    if (itype==TSTRING) table_deblank ((char*)keyvals[number]);
  } /* end search over application keywords */

  /* copy to table_def structure */
  if (tabledef->keywords!=NULL) free (tabledef->keywords);
  number++; /* actual number rather than highest index */
  tabledef->nkey = number;
  tabledef->keywords = (char**)malloc(number*sizeof(char*));
  tabledef->keytype = (int*)malloc(number*sizeof(int));
  tabledef->keycom = (char**)malloc(number*sizeof(char*));
  tabledef->keydata = (void**)malloc(number*sizeof(void*));
  for (i=0; i<number; i++) {
    tabledef->keywords[i] =(char*) malloc(strlen(keywords[i])+1);
    strcpy (tabledef->keywords[i], keywords[i]);
    tabledef->keytype[i] = keytype[i];
    tabledef->keycom[i] = (char*)malloc(strlen(keycomm[i])+1);
    strcpy (tabledef->keycom[i], keycomm[i]);
    tabledef->keydata[i] = (void*)malloc(keysize[i]);
    memcpy (tabledef->keydata[i], keyvals[i], (size_t)keysize[i]);
  } /* end loop over keywords */
} /* end get_application_keywords */

void table_deblank (char* string)
/*----------------------------------------------------------------------- */
/*   Remove trailing blanks from a string (in place)                      */
/*   Input/Output:                                                        */
/*      string       String to truncate                                   */
/*----------------------------------------------------------------------- */
{
  long i = strlen(string)-1;

  /* loop backwards replacing blanks with zeroes, */
  /* bail out at first non blank */
  while (i>=0) {
    if (string[i]==' ') string[i--] = 0;
    else break;
  }
} /* end table_deblank  */

void padcopy (char *out, char *in, int len)
/*----------------------------------------------------------------------- */
/*   copy the contents of a string with blank padding                     */
/*   the output string will contain a null character after len characters */
/*   Inputs:                                                              */
/*     out       output string pointer                                    */
/*     in        input string pointer                                     */
/*     len       number of characters to copy, if this exceeds the length */
/*               of in, any remaining characters are blank filled         */
/*----------------------------------------------------------------------- */
{
  int i, n;
  n = strlen(in);
  if (n>len) n = len;  /* don't go further than len */

  for (i=0; i<n; i++) *out++ = *in++; /* copy characters */
  for (i=n; i<len; i++) *out++ = ' ';
  *out = 0; /* final null */
} /* end padcopy */

/* Example function */
table_def* make_XXX_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a XXX table_def object                                          */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure                  .*/
/*----------------------------------------------------------------------- */
{
#include "XXX.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_XXX_table */

table_def* make_SOURCE_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a SOURCE table_def object                                       */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure                  .*/
/*----------------------------------------------------------------------- */
{
#include "SOURCE.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_SOURCE_table */

table_def* make_ARRAY_GEOMETRY_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a ARRAY_GEOMETRY table_def object                               */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure                  .*/
/*----------------------------------------------------------------------- */
{
#include "ARRAY_GEOMETRY.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_ARRAY_GEOMETRY_table */

table_def* make_ADAPTIVE_OPTICS_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a ADAPTIVE_OPTICS table_def object                              */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "ADAPTIVE_OPTICS.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_ADAPTIVE_OPTICS_table */

table_def* make_FLAG_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a FLAG table_def object                                         */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "FLAG.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_FLAG_table */

table_def* make_DELAY_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a DELAY table_def object                                        */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "DELAY.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_DELAY_table */

table_def* make_IMAGING_DATA_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a IMAGING_DATA table_def object                                 */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "IMAGING_DATA.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_IMAGING_DATA_table */

table_def* make_IMAGING_FDATA_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a IMAGING_FDATA table_def object                                */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
  /* same structure as imaging_data except data type of data */
#include "IMAGING_DATA.h"

  /* actual table type */
  char* xxxname="IMAGING_DATA";
 
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_IMAGING_FDATA_table */

table_def* make_IMAGING_DETECTOR_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a IMAGING_DETECTOR table_def object                             */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "IMAGING_DETECTOR.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_IMAGING_DETECTOR_table */

table_def* make_PREVISIBILITY_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a PREVISIBILITY table_def object                                */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "PREVISIBILITY.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_PREVISIBILITY_table */

table_def* make_VISIBILITY_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a VISIBILITY table_def object                                   */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "VISIBILITY.h"
  /* create and return structure */
  return  create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_VISIBILITY_table */

table_def* make_TIP_TILT_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a TIP_TILT table_def object                                     */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "TIP_TILT.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_TIP_TILT_table */

table_def* make_CALIBRATION_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a CALIBRATION table_def object                                  */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "CALIBRATION.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_CALIBRATION_table */

table_def* make_OPTICAL_TRAIN_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a OPTICAL_TRAIN table_def object                                */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "OPTICAL_TRAIN.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_OPTICAL_TRAIN_table */

table_def* make_WAVELENGTH_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a WAVELENGTH table_def object                             */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "OI_WAVELENGTH.h"
  /* create and return structure */
  return create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_WAVELENGTH_table */

table_def* make_OI_VIS_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a OI_VIS table_def object                                   */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "OI_VIS.h"
  /* create and return structure */
  return  create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_OI_VIS_table */

table_def* make_DISPERSION_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a DISPERSION table_def object                                   */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "DISPERSION.h"
  /* create and return structure */
  return  create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_DISPERSION_table */

table_def* make_OI_TARGET_table(char  colsymb, int symcount)
/*----------------------------------------------------------------------- */
/*   Make a OI_TARGET table_def object                                   */
/*   Inputs:                                                              */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure.                  */
/*----------------------------------------------------------------------- */
{
#include "OI_TARGET.h"
  /* create and return structure */
  nkey = 1;
  ncol = 17;
  return  create_table_def (colsymb, symcount,
			   extname, nkey, keywords, keytype, keycom,
			   ncol, coltype, colcount, coldatatype, 
			         colunits, coldim);
} /* end  make_OI_TARGET_table */
/* */
